<?php

namespace FiloBlu\Refilo\Helper\Catalog;

use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Model\ResourceModel\Eav\Attribute;
use Magento\Eav\Api\AttributeRepositoryInterface;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Swatches\Helper\Data;
use Magento\Swatches\Helper\Media;
use Magento\Swatches\Model\Swatch;

/**
 * Class ProductAttributeHelper
 * @package FiloBlu\Refilo\Helper\Catalog
 */
class ProductAttributeHelper
{
    /** @var string[] */
    const MANDATORY_ATTRIBUTES = [
        'sku',
        'name',
        'description',
        'giftcard_type',
        'allow_message'
    ];
    /**
     * TODO: attribute to exclude must be configurable
     * @var string[]
     */
    const EXCLUDED_ATTRIBUTES = [
        'created_at',
        'custom_design',
        'custom_design_from',
        'custom_design_to',
        'custom_layout',
        'custom_layout_update',
        'custom_layout_update_file',
        'email_template',
        'has_options',
        'media_gallery',
        'minimal_price',
        'price',
        'msrp_display_actual_price_type',
        'old_id',
        'image',
        'image_label',
        'links_exist',
        'links_purchased_separately',
        'links_title',
        'merchant_center_category',
        'msrp',
        'options_container',
        'origin_storage',
        'package_mode',
        'package_type',
        'page_layout',
        'price_type',
        'price_view',
        'quantity_and_stock_status',
        'related_tgtr_position_behavior',
        'related_tgtr_position_limit',
        'required_options',
        'small_image',
        'small_image_label',
        'special_price',
        'status',
        'swatch_image',
        'thumbnail',
        'thumbnail_label',
        'tier_price',
        'type_of_product',
        'updated_at',
        'upsell_tgtr_position_behavior',
        'upsell_tgtr_position_limit',
        'use_config_allow_message',
        'use_config_email_template',
        'use_config_is_redeemable',
        'visibility',
        'shipment_type'
    ];

    /** @var StoreInterface */
    protected $currentStore = null;
    /** @var array|null */
    protected $currentAttributes = null;
    /** @var array */
    protected $imagePlaceholderAttributes = [];
    /** @var array */
    protected $attributes;
    /** @var array */
    protected $filterableAttributes = [];
    /** @var array */
    protected $sortableAttributes = [];
    /** @var array */
    protected $lookupOptions = [];
    /** @var SearchCriteriaBuilder */
    private $searchCriteriaBuilder;
    /** @var AttributeRepositoryInterface */
    private $attributeRepository;
    /** @var StoreManagerInterface */
    private $storeManager;
    /** @var Data */
    private $swatchHelper;
    /** @var Media */
    private $swatchMedia;

    /**
     * ProductAttributeHelper constructor.
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param AttributeRepositoryInterface $attributeRepository
     * @param StoreManagerInterface $storeManager
     * @param Data $swatchHelper
     * @param Media $swatchMedia
     */
    public function __construct(
        SearchCriteriaBuilder $searchCriteriaBuilder,
        AttributeRepositoryInterface $attributeRepository,
        StoreManagerInterface $storeManager,
        Data $swatchHelper,
        Media $swatchMedia
    )
    {
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->attributeRepository = $attributeRepository;
        $this->storeManager = $storeManager;
        $this->swatchHelper = $swatchHelper;
        $this->swatchMedia = $swatchMedia;
    }


    /**
     *
     */
    public function getAttributes()
    {
        if ($this->attributes !== null) {
            return $this->attributes;
        }

        $searchCriteria = $this->searchCriteriaBuilder->create();
        $response = $this->attributeRepository->getList('catalog_product', $searchCriteria);

        foreach ($this->storeManager->getStores() as $store) {
            $storeId = $store->getId();
            $this->filterableAttributes[$storeId] = [];
            $this->sortableAttributes[$storeId] = [];
            $this->attributes[$storeId] = [];
        }

        foreach ($this->storeManager->getStores(true) as $store) {
            $storeId = $store->getId();
            /** @var Attribute $attribute */
            foreach ($response->getItems() as $attribute) {
                $attributeCode = $attribute->getAttributeCode();
                $isSwatch = (bool)$this->swatchHelper->isSwatchAttribute($attribute);

                if ((bool)$attribute->getIsVisibleOnFront() ||
                    (bool)$attribute->getUsedInProductListing() ||
                    $this->isMandatory($attributeCode)) {
                    $this->attributes[$storeId][$attributeCode] = [
                        'title'         => trim($attribute->getStoreLabel($store->getId())),
                        'code'          => $attributeCode,
                        'type'          => $attribute->getFrontendInput(),
                        'isSwatch'      => $isSwatch,
                        'isFilterable'  => false,
                        'isSortable'    => false,
                        'searchable'    => $attribute->getIsSearchable(),
                        'search_weight' => $attribute->getSearchWeight(),
                        'sort_weight'   => $attribute->getSortWeight($attribute->getAttributeSetId()),
                        'position'      => $attribute->getPosition(),
                        'options'       => [],
                        'value'         => []
                    ];

                    if ($attribute->getIsFilterable()) {
                        $this->attributes[$storeId][$attributeCode]['isFilterable'] = true;
                        $this->filterableAttributes[$storeId][$attributeCode] = $this->attributes[$storeId][$attributeCode];
                    }

                    if ($attribute->getUsedForSortBy()) {
                        $this->attributes[$storeId][$attributeCode]['isSortable'] = true;
                        $this->sortableAttributes[$storeId][$attributeCode] = $this->attributes[$storeId][$attributeCode];
                    }
                }

                if ($attribute->getFrontendInput() === 'media_image') {
                    $this->imagePlaceholderAttributes[$attributeCode] = $attributeCode;
                }

                /* if($attribute->getFrontend()->getConfigField('input') === 'boolean') {} */

                if ($attribute->usesSource()) {
                    foreach ($attribute->setStoreId($storeId)->getSource()->getAllOptions() as $option) {
                        $this->lookupOptions[$store->getId()][$attributeCode]['value'][$option['value']] = (string)$option['label'];
                        if (isset($this->attributes[$storeId][$attributeCode]['options'])) {
                            $optionValue = [
                                'label'  => trim((string)$option['label']),
                                'key'    => trim($option['value']),
                                'swatch' => []
                            ];

                            if ($isSwatch) {
                                $optionValue = array_merge_recursive($optionValue, $this->getSwatchByOptionId($option['value']));
                            }

                            $this->attributes[$storeId][$attributeCode]['options'][] = $optionValue;
                        }
                    }
                }
            }
        }

        return $this->attributes;
    }

    /**
     * @param ProductInterface $product
     * @return array
     */
    public function getAttributesValues(ProductInterface $product)
    {
        return $this->getAttributesFromProduct($product, $this->getAttributes(), self::EXCLUDED_ATTRIBUTES);
    }

    /**
     * @param string $attributeCode
     * @return bool
     */
    public function isMandatory(string $attributeCode): bool
    {
        return in_array($attributeCode, self::MANDATORY_ATTRIBUTES, false);
    }

    /**
     * @param $optionId
     * @return array
     */
    public function getSwatchByOptionId($optionId)
    {
        $output = [
            'swatch' => [
                'type'  => 'empty',
                'value' => ''
            ]
        ];
        $swatches = $this->swatchHelper->getSwatchesByOptionsId([$optionId]);
        if (!isset($swatches[$optionId]['type'], $swatches[$optionId]['value'])) {
            return $output;
        }

        $type = (int)$swatches[$optionId]['type'];
        $value = $swatches[$optionId]['value'];

        switch ($type) {
            case Swatch::SWATCH_TYPE_VISUAL_IMAGE:
                // Backward compatibility
                return [
                    'type'   => 'swatch',
                    'image'  => "/attribute/swatch{$value}",
                    'swatch' => [
                        'type'  => 'swatch',
                        'value' => "/attribute/swatch{$value}"
                    ]
                ];
            case Swatch::SWATCH_TYPE_VISUAL_COLOR:
                return [
                    'swatch' => [
                        'type'  => 'color',
                        'value' => $value
                    ]
                ];
            case Swatch::SWATCH_TYPE_TEXTUAL:
                return [
                    'swatch' => [
                        'type'  => 'textual',
                        'value' => $value
                    ]
                ];
            case Swatch::SWATCH_TYPE_EMPTY:
            default:
                return $output;
        }
    }

    /**
     * @param ProductInterface $product
     * @param array $attributeList
     * @param array $excludeList
     * @return array
     */
    protected function getAttributesFromProduct(ProductInterface $product, array $attributeList, array $excludeList = []): array
    {
        $storeId = $product->getStore()->getId();

        $attributes = [];

        foreach ($attributeList[$storeId] as $code => $attribute) {
            if (in_array($code, $excludeList, true)) {
                continue;
            }

            $type = $attribute['type'];

            switch ($type) {
                case 'select':
                    $attributes[$code] = [
                        'title' => $attribute['title'],
                        'value' => $this->lookupOptions[$storeId][$code]['value'][$product->getData($code)] ?? null,
                        'key'   => $product->getData($code)
                    ];

                    if ($attribute['isSwatch']) {
                        $data = $this->getSwatchByOptionId($product->getData($code));
                        $attributes[$code] = array_merge_recursive($attributes[$code], $data);
                    }

                    break;
                case 'multiselect':
                    $value=[];

                    $attributeData = explode(',', $product->getData($code) ?? '');

                    if (is_scalar($attributeData)) {
                        $attributeData=[$attributeData];
                    }

                    $keys = array_map('intval', array_filter($attributeData, 'strlen'));

                    foreach ($keys as $k) {
                        if (isset($this->lookupOptions[$storeId][$code]['value'][$k])) {
                            $value[] = $this->lookupOptions[$storeId][$code]['value'][$k];
                        }
                    }

                    $attributes[$code] = [
                        'title' => $attribute['title'],
                        'value' => $value,
                        'key'   => $keys
                    ];
                    break;
                default:
                    $attributes[$code] = [
                        'value' => $product->getData($code),
                        'title' => $attribute['title']
                    ];
                    break;
            }
        }

        return $attributes;
    }

    /**
     * @param ProductInterface $product
     * @return array
     */
    public function getFilterableAttributes(ProductInterface $product): array
    {
        $this->getAttributes();
        return $this->getAttributesFromProduct($product, $this->filterableAttributes);
    }

    /**
     * @param ProductInterface $product
     * @return array
     */
    public function getSortableAttributes(ProductInterface $product)
    {
        $this->getAttributes();
        return $this->getAttributesFromProduct($product, $this->sortableAttributes);
    }

    /**
     * @param ProductInterface $product
     * @return array
     */
    public function getImagePlaceholdersAttributes(ProductInterface $product): array
    {
        $this->getAttributes();

        $placeholders = [];

        foreach ($this->imagePlaceholderAttributes as $attributeCode) {
            $value = $product->getData($attributeCode);
            if ($value === null) {
                continue;
            }
            $placeholders[$attributeCode] = "/catalog/product{$value}";
        }

        return $placeholders;
    }

    /**
     * @param $optionId
     * @param $storeId
     * @param $attributeCode
     * @return mixed|null
     */
    public function getProductOptionLabel($optionId, $storeId, $attributeCode)
    {
        $attributes = $this->getAttributes();

        if (!isset($attributes[$storeId][$attributeCode]['options'])){
            return null;
        }

        foreach ($attributes[$storeId][$attributeCode]['options'] as $option){
            if ($option['key'] == $optionId){
                return $option['label'];
            }
        }
        return null;
    }
}
